@@ -5,14 +5,26 @@ use 5.010_001;
 
 use strictures 2;
 
+use base 'Exporter';
+our @EXPORT_OK = qw(
+  compare_values normalize_aoh reftype
+);
+
 use Scalar::Util ();
 
+sub compare_values {
+  my ($v1, $v2) = @_;
+
+  return 1 if !defined($v1) && !defined($v2);
+  return 1 if defined($v1) && defined($v2) && $v1 eq $v2;
+  return;
+}
+
 sub reftype {
   return Scalar::Util::reftype($_[0]) // '';
 }
 
 sub normalize_aoh {
-  shift;
   my ($input) = @_;
 
   return unless defined $input;
